/*
    UHU-menu -- output plugin for Xfce (4.2)
    Copyright (C) 2005  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

static void indent (FILE *f, int n)
{
	int i;
	for (i = 1; i < n; i++) fprintf(f, "\t");
}

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		switch (*s) {
		  case '\\':
			fprintf(f, "\\\\");
			break;
		  case '/':
			fprintf(f, "\\/");
			break;
		  case '\"':
			fprintf(f, "&quot;");
			break;
		  case '<':
			fprintf(f, "&lt;");
			break;
		  case '>':
			fprintf(f, "&gt;");
			break;
		  case '&':
			fprintf(f, "&amp;");
			break;
		  default:
			fputc(*s, f);
		}
		s++;
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s == '%') {
			s++;
			if (*s == '%' || *s == '\0') fputc('%', f);
			if (*s != '\0') s++;
			continue;
		}
		switch (*s) {
		  case '\\':
			fprintf(f, "\\\\");
			break;
		  case '/':
			fprintf(f, "\\/");
			break;
		  case '\"':
			fprintf(f, "&quot;");
			break;
		  case '<':
			fprintf(f, "&lt;");
			break;
		  case '>':
			fprintf(f, "&gt;");
			break;
		  case '&':
			fprintf(f, "&amp;");
			break;
		  default:
			fputc(*s, f);
		}
		s++;
	}
}

static void dump (FILE *f, const struct item *item, const char *lang, int n)
{
	int i;
	int major;
	char *iconstr;
	if (!item->issubmenu) {
		indent(f, n);
		fprintf(f, "<app name=\"");
		print_name(f, get_value(item, NULL, "Name", lang));
		fprintf(f, "\" cmd=\"");
		print_exec(f, get_value(item, NULL, "Exec", lang));
		fprintf(f, "\"");
		if (item->terminal) fprintf(f, " term=\"true\"");
		iconstr = get_value(item, NULL, "Icon", lang);
		if (iconstr != NULL) fprintf(f, " icon=\"%s\"", iconstr);
		fprintf(f, "/>\n");
	} else {
		if (n > 1) {
			indent(f, n);
			fprintf(f, "<menu name=\"");
			print_name(f, get_value(item, NULL, "Name", lang));
			fprintf(f, "\"");
			iconstr = get_value(item, NULL, "Icon", lang);
			if (iconstr != NULL) fprintf(f, " icon=\"%s\"", iconstr);
			fprintf(f, ">\n");
		}
		major = 0;
		for (i = 0; i < item->subitems_nr; i++) {
			if (i > 0 && item->subitems[i].major != major) {
				indent(f, n+1);
				fprintf(f, "<separator/>\n");
			}
			dump(f, &item->subitems[i], lang, n+1);
			major = item->subitems[i].major;
		}
		if (n > 1) {
			indent(f, n);
			fprintf(f, "</menu>\n");
		}
	}
}

int write_menu_lang (const struct item *item, const char *path,
  const struct language *language, struct keep_files *keep_files)
{
	char *lang;
	char file[PATH_MAX];
	char file_tmp[PATH_MAX];
	FILE *f;

	lang = language->name;
	sprintf(file, "%s/menu.xml%s%s", path, lang[0] ? "." : "", lang);
	sprintf(file_tmp, "%s..tmp", file);
	if ((f = fopen(file_tmp, "w")) == NULL) {
		message_e(3, "fopen %s", file_tmp);
		return -1;
	}
	fprintf(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	fprintf(f, "<!DOCTYPE xfdesktop-menu>\n");
	fprintf(f, "<xfdesktop-menu>\n");
	dump(f, item, lang, 1);
	fprintf(f, "</xfdesktop-menu>\n");
	fclose(f);
	move(file_tmp, file);
	keep_file(keep_files, strrchr(file, '/') + 1);
	return 0;
}

